package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeHISDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.dto.StaticKnowledgeIndexPageDTO;
import com.diagbot.entity.SysUser;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.PushJoinVO;
import com.diagbot.vo.ScaleStaticAllVO;
import com.diagbot.vo.ScaleVO;
import com.diagbot.vo.SearchData;
import com.diagbot.vo.StaticKnowledgeHISVO;
import com.diagbot.vo.StaticKnowledgeIndexPageVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.diagbot.vo.StaticKnowledgeVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/23 15:41
 */
@Component
public class KlConceptStaticFacade {

    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private ConceptInfoFacade conceptInfoFacade;
    @Autowired
    private SysUserFacade sysUserFacade;
    @Autowired
    MrFacade mrFacade;
    @Autowired
    private AssembleFacade assembleFacade;


    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();

        RespDTO<List<StaticKnowledgeIndexDTO>> respDTO = cdssCoreClient.staticKnowledgeIndex(staticKnowledgeIndexVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            retList = respDTO.data;
        }

        if (staticKnowledgeIndexVO.getHasInfo() != null && staticKnowledgeIndexVO.getHasInfo().equals(1)) {
            retList = retList.stream().filter(i -> i.getStatus().equals(1)).collect(Collectors.toList());
        }
        return retList;
    }


    /**
     * 获取静态知识
     *
     * @param staticKnowledgeVO
     * @return
     */
    public StaticKnowledgeDTO getStaticKnowledge(StaticKnowledgeVO staticKnowledgeVO) {
        ScaleStaticAllVO scaleStaticAllVO = new ScaleStaticAllVO();
        if (staticKnowledgeVO.getType() == 8) {
            PushJoinVO mr = mrFacade.getMr(staticKnowledgeVO.getMrId());
            if (null != mr) {
                SearchData searchData = new SearchData();
                BeanUtil.copyProperties(mr, searchData);
                //入参映射
                searchData = assembleFacade.assembleData(searchData);
                BeanUtil.copyProperties(searchData, scaleStaticAllVO);
            }
        } else {
            scaleStaticAllVO.setHospitalId(-1L);
        }
        BeanUtil.copyProperties(staticKnowledgeVO, scaleStaticAllVO);
        RespDTO<StaticKnowledgeDTO> respDTO = cdssCoreClient.getStaticKnowledge(scaleStaticAllVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            StaticKnowledgeDTO staticKnowledgeDTO = respDTO.data;
            return staticKnowledgeDTO;
        } else {
            return null;
        }
    }

    /**
     * 医院端获取静态知识（对接）
     *
     * @param staticKnowledgeHISVO
     * @return
     */
    public List<StaticKnowledgeHISDTO> getStaticKnowledgeForHIS(StaticKnowledgeHISVO staticKnowledgeHISVO) {

        List<StaticKnowledgeHISDTO> retList = Lists.newArrayList();

        //术语映射
        List<String> uniqueNameList = conceptInfoFacade.getUniqueNames(staticKnowledgeHISVO);
        if (ListUtil.isEmpty(uniqueNameList)) {
            return retList;
        }

        for (String uniqueName : uniqueNameList) {
            StaticKnowledgeVO staticKnowledgeVO = new StaticKnowledgeVO();
            BeanUtil.copyProperties(staticKnowledgeHISVO, staticKnowledgeVO);
            staticKnowledgeVO.setName(uniqueName);
            StaticKnowledgeDTO staticKnowledgeDTO = getStaticKnowledge(staticKnowledgeVO);
            if (staticKnowledgeDTO != null) {
                StaticKnowledgeHISDTO staticKnowledgeHISDTO = new StaticKnowledgeHISDTO();
                BeanUtil.copyProperties(staticKnowledgeDTO, staticKnowledgeHISDTO);
                staticKnowledgeHISDTO.setHisName(staticKnowledgeHISVO.getHisName());
                staticKnowledgeHISDTO.setHisDetailName(staticKnowledgeHISVO.getHisDetailName());
                retList.add(staticKnowledgeHISDTO);
            }
        }
        return retList;
    }

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    public IPage<KlConceptStaticDTO> getPage(KlConceptStaticPageVO klConceptStaticPageVO) {
        Page<KlConceptStaticDTO> page = null;
        RespDTO<Page<KlConceptStaticDTO>> respDTO = cdssCoreClient.getPage(klConceptStaticPageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        }
        List<KlConceptStaticDTO> records = page.getRecords();
        List<Long> userIds = records.stream().map(KlConceptStaticDTO::getModifier)
                .distinct()
                .map(Long::parseLong)
                .collect(Collectors.toList());
        List<SysUser> users = sysUserFacade.list(new QueryWrapper<SysUser>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("id", userIds, false));
        if (ListUtil.isNotEmpty(users)) {
            Map<Long, String> userMap = users.stream().collect(Collectors.toMap(SysUser::getId, SysUser::getLinkman));
            records.forEach(record -> {
                if (userMap.containsKey(Long.valueOf(record.getModifier()))) {
                    record.setModifier(userMap.get(Long.valueOf(record.getModifier())));
                } else {
                    record.setModifier("");
                }
            });

        } else {
            records.forEach(record -> {
                record.setModifier("");
            });
        }
        page.setRecords(records);
        return page;
    }

    /**
     * 分页查询
     *
     * @param staticKnowledgeIndexPageVO
     * @return
     */
    public IPage<StaticKnowledgeIndexPageDTO> staticIndexPage(StaticKnowledgeIndexPageVO staticKnowledgeIndexPageVO) {
        Page<StaticKnowledgeIndexPageDTO> page = null;

        RespDTO<Page<StaticKnowledgeIndexPageDTO>> respDTO = cdssCoreClient.staticIndexPage(staticKnowledgeIndexPageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        }
        return page;
    }

    /**
     * 保存记录（新增or修改）
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean saveOrUpdateRecord(KlConceptStaticVO klConceptStaticVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        klConceptStaticVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.saveOrUpdateRecord(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        }
        return success;
    }

    /**
     * 启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeStatus(ChangeStatusVO changeStatusVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        changeStatusVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.changeStatus(changeStatusVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        }
        return success;
    }

    /**
     * 根据术语id获取静态信息
     *
     * @param idVO
     * @return
     */
    public KlConceptStaticDTO getRecordById(IdVO idVO) {
        KlConceptStaticDTO klConceptStaticDTO = new KlConceptStaticDTO();
        RespDTO<KlConceptStaticDTO> respDTO = cdssCoreClient.getRecordById(idVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            klConceptStaticDTO = respDTO.data;
        }
        return klConceptStaticDTO;
    }

    /**
     * 是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean isExist(KlConceptStaticVO klConceptStaticVO) {
        Boolean exist = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.isExist(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            exist = respDTO.data;
        }
        return exist;
    }
}