package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.NurseInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.NurseConfig;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.NurseConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.NurseConfigListVO;
import com.diagbot.vo.NurseConfigPageVO;
import com.diagbot.vo.RetrievalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/4/26 10:31
 */
@Component
public class NurseConfigFacade {
    @Autowired
    private NurseConfigService nurseConfigService;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 判断是否已存在
     *
     * @param nurseConfig
     * @return
     */
    public Boolean isExistRecord(NurseConfig nurseConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<NurseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", nurseConfig.getHisName())
                .eq("unique_name", nurseConfig.getUniqueName());
        NurseConfig oldRecord = nurseConfigService.getOne(queryWrapper, false);
        if (nurseConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (nurseConfig.getId() != null
                && oldRecord != null
                && !nurseConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param nurseConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(NurseConfig nurseConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        nurseConfig.setHospitalId(Long.valueOf(hospitalId));
        nurseConfig.setModifier(userId);
        nurseConfig.setGmtModified(now);
        QueryWrapper<NurseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", nurseConfig.getHisName())
                .eq("unique_name", nurseConfig.getUniqueName());
        NurseConfig oldRecord = nurseConfigService.getOne(queryWrapper, false);
        if (nurseConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (nurseConfig.getId() != null
                && oldRecord != null
                && !nurseConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (nurseConfig.getId() == null) {
            nurseConfig.setCreator(userId);
            nurseConfig.setGmtCreate(now);
        }
        if (nurseConfig.getIsDeleted() == null) {
            nurseConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        nurseConfigService.saveOrUpdate(nurseConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param nurseConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(NurseConfigListVO nurseConfigListVO) {
        if (ListUtil.isEmpty(nurseConfigListVO.getNurseConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(nurseConfigListVO.getNurseConfigList());
    }

    /**
     * 批量保存
     *
     * @param nurseConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<NurseConfig> nurseConfigList) {
        if (ListUtil.isEmpty(nurseConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        nurseConfigList = nurseConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, List<Long>>> configMap
                = getConfigMapWithIds(Long.valueOf(hospitalId), null, null);
        nurseConfigList.forEach(nurseConfig -> {
            nurseConfig.setHospitalId(Long.valueOf(hospitalId));
            nurseConfig.setModifier(userId);
            nurseConfig.setGmtModified(now);
            if (nurseConfig.getId() == null) {
                if (configMap.containsKey(nurseConfig.getHisName())
                        &&ListUtil.isNotEmpty(configMap.get(nurseConfig.getHisName()).get(nurseConfig.getUniqueName()))) {
                    deleteIds.addAll(configMap.get(nurseConfig.getHisName()).get(nurseConfig.getUniqueName()));
                }
                nurseConfig.setCreator(userId);
                nurseConfig.setGmtCreate(now);
            }
            if (nurseConfig.getIsDeleted() == null) {
                nurseConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        nurseConfigService.saveOrUpdateBatch(nurseConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<NurseConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        nurseConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<NurseConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        nurseConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param nurseConfigPageVO
     * @return
     */
    public IPage<NurseConfig> getPage(NurseConfigPageVO nurseConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        nurseConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return nurseConfigService.getPage(nurseConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        List<NurseConfig> nurseConfigList = ExcelUtils.importExcel(file, 0, 1, NurseConfig.class);
        if (ListUtil.isNotEmpty(nurseConfigList)) {
            importExcelRecords(nurseConfigList);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导入
     *
     * @param nurseConfigList
     * @return
     */
    public Boolean importExcelRecords(List<NurseConfig> nurseConfigList) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //1、数据完整性校验
        //2、去除前后空格
        //过滤空数据，保留重复数据，方便计行
        nurseConfigList = nurseConfigList.stream()
                .filter(NurseConfig::nonNull)
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(nurseConfigList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<String> emptyNumList = Lists.newLinkedList();
        for (int i = 0; i < nurseConfigList.size(); i++) {
            if (StringUtil.isBlank(nurseConfigList.get(i).getHisName())
                    || StringUtil.isBlank(nurseConfigList.get(i).getUniqueName())) {
                emptyNumList.add(String.valueOf(i + 2));
            }
            if (StringUtil.isNotBlank(nurseConfigList.get(i).getHisName())) {
                nurseConfigList.get(i).setHisName(nurseConfigList.get(i).getHisName().trim());
            }
            if (StringUtil.isNotBlank(nurseConfigList.get(i).getUniqueName())) {
                nurseConfigList.get(i).setUniqueName(nurseConfigList.get(i).getUniqueName().trim());
            }
            if (StringUtil.isNotBlank(nurseConfigList.get(i).getUniqueCode())) {
                nurseConfigList.get(i).setUniqueCode(nurseConfigList.get(i).getUniqueCode().trim());
            } else {
                nurseConfigList.get(i).setUniqueCode(null);
            }
        }

        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        synchronized (this) {
            List<Long> deleteIds = Lists.newLinkedList();
            Map<String, Map<String, List<Long>>> configMap
                    = getConfigMapWithIds(Long.valueOf(hospitalId), null, null);
            nurseConfigList.forEach(nurseConfig -> {
                nurseConfig.setHospitalId(Long.valueOf(hospitalId));
                nurseConfig.setModifier(userId);
                nurseConfig.setGmtModified(now);
                if (nurseConfig.getId() == null) {
                    if (configMap.containsKey(nurseConfig.getHisName())
                            && ListUtil.isNotEmpty(configMap.get(nurseConfig.getHisName()).get(nurseConfig.getUniqueName()))) {
                        deleteIds.addAll(configMap.get(nurseConfig.getHisName()).get(nurseConfig.getUniqueName()));
                    }
                    nurseConfig.setCreator(userId);
                    nurseConfig.setGmtCreate(now);
                }
                if (nurseConfig.getIsDeleted() == null) {
                    nurseConfig.setIsDeleted(IsDeleteEnum.N.getKey());
                }
            });

            //标准术语校验
            List<String> errorNumList = Lists.newLinkedList();
            List<String> uniqueNames = nurseConfigList.stream()
                    .map(i -> i.getUniqueName())
                    .distinct()
                    .collect(Collectors.toList());
            ConceptVO conceptVO = new ConceptVO();
            conceptVO.setNames(uniqueNames);
            conceptVO.setType(ConceptTypeEnum.Nurse.getKey());
            RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
            RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
            List<String> names = respDTO.data.stream().map(IndexBatchDTO::getName).collect(Collectors.toList());
            for (int i = 0; i < nurseConfigList.size(); i++) {
                if (!names.contains(nurseConfigList.get(i).getUniqueName())) {
                    errorNumList.add(String.valueOf(i + 2));
                }
            }
            if (ListUtil.isNotEmpty(errorNumList)) {
                throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                        "以下行数（不计空行）标准术语在数据库中不存在："
                                + errorNumList.stream().collect(Collectors.joining("、"))
                                + "。导入取消，请修改后再试。");
            }

            //重复数据过滤
            nurseConfigList = nurseConfigList
                    .stream()
                    .distinct()
                    .collect(Collectors.toList());

            //删除已存在映射关系
            IdListVO idListVO = new IdListVO();
            idListVO.setIds(deleteIds);
            deleteRecords(idListVO);
            nurseConfigService.saveOrUpdateBatch(nurseConfigList);
        }
        return true;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<NurseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<NurseConfig> records = nurseConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<NurseConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<NurseConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, List<Long>>> getConfigMapWithIds(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, List<Long>>> retMap = new HashMap<>();
        QueryWrapper<NurseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<NurseConfig> records = nurseConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<NurseConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<NurseConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, List<NurseConfig>> subMap = EntityUtil.makeEntityListMap(entry.getValue(), "uniqueName");
                Map<String, List<Long>> subIdMap = new HashMap<>();
                for (Map.Entry<String, List<NurseConfig>> subEntry : subMap.entrySet()) {
                    subIdMap.put(subEntry.getKey(), subEntry.getValue().stream().map(i -> i.getId()).distinct().collect(Collectors.toList()));
                }
                retMap.put(entry.getKey(), subIdMap);
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<NurseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<NurseConfig> records = nurseConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        Map<String, List<NurseConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<NurseConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "hisName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<NurseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID())
                .orderByDesc("gmt_modified");
        List<NurseConfig> records = nurseConfigService.list(queryWrapper);
        String fileName = "护理映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", NurseConfig.class, fileName, response, 12.8f);
    }

    /**
     * 数据导入模板导出
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response) {
        String fileName = "护理映射模板.xls";
        ExcelUtils.exportExcel(new ArrayList<>(), null, "sheet1", NurseConfig.class, fileName, response, 12.8f);
    }

    /**
     * 手术搜索
     *
     * @param retrievalVO
     * @return
     */
    public List<NurseInfoDTO> getNurses(RetrievalVO retrievalVO) {
        List<NurseInfoDTO> nurseNames = new ArrayList<>();
        List<NurseConfig> nurseConfigList = nurseConfigService.getNursesIndex(retrievalVO);
        for (NurseConfig nurseConfig: nurseConfigList){
            NurseInfoDTO nurseInfoDTO = new NurseInfoDTO();
            nurseInfoDTO.setName(nurseConfig.getHisName());
            nurseNames.add(nurseInfoDTO);
        }
        return nurseNames;
    }
}
