package com.diagbot.facade;

import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.HospitalCheckVO;
import com.diagbot.vo.RetrievalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:44
 */
@Component
public class RetrievalFacade {
    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private HospitalInfoFacade hospitalInfoFacade;
    @Autowired
    private LisConfigFacade lisConfigFacade;
    @Autowired
    private PacsConfigFacade pacsConfigFacade;
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;
    @Autowired
    private DrugConfigFacade drugConfigFacade;
    @Autowired
    private OperationConfigFacade operationConfigFacade;
    @Autowired
    private DeptConfigFacade deptConfigFacade;
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;
    @Autowired
    private ScaleConfigFacade scaleConfigFacade;
    @Autowired
    private NurseConfigFacade nurseConfigFacade;
    @Autowired
    private TcmdiseaseConfigFacade tcmdiseaseConfigFacade;
    @Autowired
    private TcmsyndromeConfigFacade tcmsyndromeConfigFacade;

    /**
     * 检索
     *
     * @param retrievalVO
     * @return
     */
    public RetrievalDTO index(RetrievalVO retrievalVO) {
        RetrievalDTO retrievalDTO = new RetrievalDTO();
        if (null == retrievalVO
                || (null != retrievalVO
                && (null == retrievalVO.getHospitalId() || retrievalVO.getHospitalId().equals(-1L)))) {
            //无医院或者朗通医院查询标准词
            RespDTO<RetrievalDTO> respDTO = cdssCoreClient.index(retrievalVO);
            RespDTOUtil.respNGDealCover(respDTO, "检索失败");
            retrievalDTO = respDTO.data;
        } else {
            //实际医院查询医院词
            if (StringUtil.isBlank(retrievalVO.getInputStr())) {
                throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入检索内容！");
            }
            HospitalCheckVO hospitalCheckVO = new HospitalCheckVO();
            hospitalCheckVO.setHospitalId(retrievalVO.getHospitalId());
            if (!hospitalInfoFacade.checkHospital(hospitalCheckVO)) {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "该医院ID不存在或者未启用！");
            } else {
                switch (retrievalVO.getType()) {
                    case 1:
                        retrievalDTO.setLisNames(lisConfigFacade.getLisPacks(retrievalVO));
                        break;
                    case 2:
                        retrievalDTO.setLisDetailNames(lisConfigFacade.getLis(retrievalVO));
                        break;
                    case 3:
                        retrievalDTO.setPacsNames(pacsConfigFacade.getPACSs(retrievalVO));
                        break;
                    case 4:
                        retrievalDTO.setDiseaseNames(diseaseConfigFacade.getDiseases(retrievalVO));
                        break;
                    case 5:
                        retrievalDTO.setDrugNames(drugConfigFacade.getMedicines(retrievalVO));
                        break;
                    case 6:
                        retrievalDTO.setOperationNames(operationConfigFacade.getOperations(retrievalVO));
                        break;
                    case 7:
                        retrievalDTO.setDeptNames(deptConfigFacade.getDepts(retrievalVO));
                        break;
                    case 8:
                        retrievalDTO.setTransfusionNames(transfusionConfigFacade.getTransfusion(retrievalVO));
                        break;
                    case 10:
                        retrievalDTO.setScalenames(scaleConfigFacade.getScales(retrievalVO));
                        break;
                    case 11:
                        retrievalDTO.setNursenames(nurseConfigFacade.getNurses(retrievalVO));
                        break;
                    case 12:
                        retrievalDTO.setTcmdiseaseNames(tcmdiseaseConfigFacade.getTcmdiseases(retrievalVO));
                        break;
                    case 13:
                        retrievalDTO.setTcmsyndromeNames(tcmsyndromeConfigFacade.getTcmsyndromes(retrievalVO));
                        break;
                }
            }
        }
        return retrievalDTO;
    }
}