package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.TcmdiseaseConfig;
import com.diagbot.facade.TcmdiseaseConfigFacade;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.TcmdiseaseConfigListVO;
import com.diagbot.vo.TcmdiseaseConfigPageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * <p>
 * 中医疾病映射表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2021-05-13
 */
@RestController
@RequestMapping("/tran/tcmdiseaseConfig")
@Api(value = "中医疾病公表映射API", tags = { "中医疾病公表映射API" })
public class TcmdiseaseConfigController {
    @Autowired
    private TcmdiseaseConfigFacade tcmdiseaseConfigFacade;

    /**
     * 映射关系是否已存在
     *
     * @param tcmdiseaseConfig
     * @return
     */
    @ApiOperation(value = "映射关系是否已存在[by:zhaops]", notes = "")
    @PostMapping("/isExistRecord")
    @SysLogger("isExistRecord")
    public RespDTO<Boolean> isExistRecord(@RequestBody @Valid TcmdiseaseConfig tcmdiseaseConfig) {
        Boolean data = tcmdiseaseConfigFacade.isExistRecord(tcmdiseaseConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 保存或修改映射关系
     *
     * @param tcmdiseaseConfig
     * @return
     */
    @ApiOperation(value = "保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@RequestBody @Valid TcmdiseaseConfig tcmdiseaseConfig) {
        Boolean data = tcmdiseaseConfigFacade.saveOrUpdateRecord(tcmdiseaseConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量保存或修改映射关系
     *
     * @param tcmdiseaseConfigListVO
     * @return
     */
    @ApiOperation(value = "批量保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecords")
    @SysLogger("saveOrUpdateRecords")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecords(@RequestBody @Valid TcmdiseaseConfigListVO tcmdiseaseConfigListVO) {
        Boolean data = tcmdiseaseConfigFacade.saveOrUpdateRecords(tcmdiseaseConfigListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 删除映射关系
     *
     * @param idVO
     * @return
     */
    @ApiOperation(value = "删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecord")
    @SysLogger("deleteRecord")
    @Transactional
    public RespDTO<Boolean> deleteRecord(@RequestBody @Valid IdVO idVO) {
        Boolean data = tcmdiseaseConfigFacade.deleteRecord(idVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量删除映射关系
     *
     * @param idListVO
     * @return
     */
    @ApiOperation(value = "批量删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecords")
    @SysLogger("deleteRecords")
    @Transactional
    public RespDTO<Boolean> deleteRecords(@RequestBody @Valid IdListVO idListVO) {
        Boolean data = tcmdiseaseConfigFacade.deleteRecords(idListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 分页查询
     *
     * @param tcmdiseaseConfigPageVO
     * @return
     */
    @ApiOperation(value = "分页查询[by:zhaops]", notes = "")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<TcmdiseaseConfig> getPage(@RequestBody @Valid TcmdiseaseConfigPageVO tcmdiseaseConfigPageVO) {
        IPage<TcmdiseaseConfig> data = tcmdiseaseConfigFacade.getPage(tcmdiseaseConfigPageVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 公表数据导入
     *
     * @param file
     * @return
     */
    @ApiOperation(value = "公表数据导入[by:zhaops]",
            notes = "")
    @PostMapping("/importExcel")
    @SysLogger("importExcel")
    @Transactional
    public void importExcel(@RequestParam("file") MultipartFile file) {
        tcmdiseaseConfigFacade.importExcel(file);
    }

    /**
     * 数据导出
     *
     * @return
     */
    @ApiOperation(value = "数据导出[by:gaodm]",
            notes = "")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response) {
        tcmdiseaseConfigFacade.exportExcel(response);
    }

    /**
     * 数据导入模板导出
     *
     * @return
     */
    @ApiOperation(value = "数据导入模板导出[by:zhaops]",
            notes = "")
    @PostMapping("/exportExcelModule")
    @SysLogger("exportExcelModule")
    public void exportExcelModule(HttpServletResponse response) {
        tcmdiseaseConfigFacade.exportExcelModule(response);
    }
}
